

#if !defined __DISCOORD_H
#define __DISCOORD_H


#include "matrix.h"
#include "dataio.h"


//#define DIS_PDU_VERSION 3 // May 93 version, draft 3
#define DIS_PDU_VERSION 5   // 1995 version

#define DIS_PDU_FAMILY_ENTITY_INFO_INTERACT  1
#define DIS_PDU_FAMILY_SIM_MANAGEMENT        5
#define DIS_PDU_FAMILY_EMISSION_REGENERATION 6

#define DIS_PDU_TYPE_ENTITY_STATE     1
#define DIS_PDU_TYPE_FIRE             2
#define DIS_PDU_TYPE_DETONATE         3
#define DIS_PDU_TYPE_CREATE_ENTITY   11
#define DIS_PDU_TYPE_REMOVE_ENTITY   12
#define DIS_PDU_TYPE_ACTION_REQUEST  16
#define DIS_PDU_TYPE_ACTION_RESPONSE 17
#define DIS_PDU_TYPE_EMISSIONS       23
//#define DIS_PDU_TYPE_ACTION_REQUEST  56
//#define DIS_PDU_TYPE_ACTION_RESPONSE 57

#define DIS_PDU_SIZE_BASE_ENTITY_STATE         144 //  1152
#define DIS_PDU_SIZE_FIRE                       96 //   768
#define DIS_PDU_SIZE_BASE_DETONATE             104 //   832
#define DIS_PDU_SIZE_BASE_ACTION_REQUEST        40 //   320
#define DIS_PDU_SIZE_BASE_ACTION_RESPONSE       40 //   320
#define DIS_PDU_SIZE_BASE_EMISSIONS_TARGET       8 //    64
#define DIS_PDU_SIZE_BASE_EMISSIONS_TARGET_32    2
#define DIS_PDU_SIZE_BASE_EMISSIONS_BEAM        52 //   416
#define DIS_PDU_SIZE_BASE_EMISSIONS_BEAM_32     13
#define DIS_PDU_SIZE_BASE_EMISSIONS_SYSTEM      20 //   160
#define DIS_PDU_SIZE_BASE_EMISSIONS_SYSTEM_32    5
#define DIS_PDU_SIZE_BASE_EMISSIONS             28 //   224
//#define DIS_PDU_SIZE_MAX_PDU_SIZE             8192 // 65535
#define DIS_PDU_SIZE_MAX_PDU_SIZE             8000 // 64000

#define DIS_PDU_APPEARANCE_DESTROYED 0x00800000

#define DIS_POSITION 0
#define DIS_VECTOR   1

// orientation - Rotate/Fixed
// position - Position rates (velocity) / Velocity rates (acceleration)
// World coords / Body coords
 
#define DIS_DR_OTHER 0
#define DIS_DR_FIXED 1
#define DIS_DR_FPW   2
#define DIS_DR_RPW   3
#define DIS_DR_RVW   4
#define DIS_DR_FVW   5
#define DIS_DR_FPB   6
#define DIS_DR_RPB   7
#define DIS_DR_RVB   8
#define DIS_DR_FVB   9

#define DIS_KIND_PLATFORM    1
#define DIS_KIND_MUNITION    2
#define DIS_KIND_LIFE        3
#define DIS_KIND_ENVIRONMENT 4
#define DIS_KIND_FEATURE     5


#define DIS_KIND_DOMAIN_LAND       1
#define DIS_KIND_DOMAIN_AIR        2
#define DIS_KIND_DOMAIN_SURFACE    3
#define DIS_KIND_DOMAIN_SUBSURFACE 4
#define DIS_KIND_DOMAIN_SPACE      5
#define DIS_KIND_DOMAIN_SUPPLY     6
#define DIS_KIND_DOMAIN_RADIO      7
#define DIS_KIND_DOMAIN_EXPENDABLE 8
#define DIS_KIND_DOMAIN_SENSOR     9

#define DIS_COUNTRY_USA 225

#define DIS_FORCE_OTHER     0
#define DIS_FORCE_FRIENDLY  1
#define DIS_FORCE_OPPOSING 2
#define DIS_FORCE_NEUTRAL   3

#define FIRE_PDU_NO_FIRE_MISSION 0

#define DETONATION_NORMAL 1
#define DETONATION_HE     5
#define DETONATION_NONE   6

#define DIS_EMISSION_STATE_UPDATE 0
#define DIS_EMISSION_STATE_CHANGE 1

#define DIS_EMITTER_NAME_OTHER 62020

#define DIS_EMITTER_FUNCTION_OTHER 0

#define DIS_BEAM_FUNCTION_OTHER 0
#define DIS_BEAM_FUNCTION_SEARCH 1
#define DIS_BEAM_FUNCTION_AQUISITION 3
#define DIS_BEAM_FUNCTION_TRACK 4
#define DIS_BEAM_FUNCTION_GUIDANCE 6

#define DIS_TRACK_JAM_NOT_SELECTED 0
#define DIS_TRACK_JAM_SELECTED     1


typedef struct {

   double latitude;   // radians
   double longitude;  // radians
   double altitude;   // meters

} lla_type;


typedef struct {

   int entity;
   int site;
   int application;
} DIS_id_type;


/* **********************************************************
********************************************************** */
class DIS_header_type {

   public:
      char version;
      char exercise_id;
      char type;
      char family;
      unsigned int time;
      unsigned short size;
      unsigned short padd;

      DIS_header_type();
      virtual ~DIS_header_type() {}
   
      void build(sfile *sbuffer);
      void extract(sfile *sbuffer);
};


/* **********************************************************
********************************************************** */
class DIS_entity_type_type {

   public:
      char kind;
      char domain;
      unsigned short country;
      char category;
      char subcategory;
      char specific;
      char extra;

      virtual ~DIS_entity_type_type() {}
      
      void build(sfile *sbuffer);
      void extract(sfile *sbuffer);
};


/* **********************************************************
********************************************************** */
class DIS_burst_descriptor_type {

   public:
      DIS_entity_type_type burst_id;
      unsigned short burst_warhead;
      unsigned short burst_fuze;
      unsigned short burst_quantity;
      unsigned short burst_rate;

      virtual ~DIS_burst_descriptor_type() {};

      void build(sfile *sbuffer);
      void extract(sfile *sbuffer);
};


/* **********************************************************
********************************************************** */
class DISPDU_entity_state_type {

   public:
      DIS_header_type header;

      unsigned short site_id;
      unsigned short application_id;
      unsigned short entity_id;

      char force_id;

      char articulation_parameter_count;

      DIS_entity_type_type entity_type;
      DIS_entity_type_type alt_entity_type;

      vector3f velocity;

      vector3d location;
   
      vector3f ypr;

      unsigned int appearance_flag;

      char dr_algorithm;
      char dr_other[15];
      vector3f dr_accel;
      vector3f dr_ypr_velocity;

      char character_set;
      unsigned char marking[11];

      unsigned int capability_flag;

      DISPDU_entity_state_type();
      virtual ~DISPDU_entity_state_type() {}

      void setup(int id, int fid, DIS_entity_type_type *type, int dr_algo);
      void build(buffer_type *buffer);
      void extract(buffer_type *buffer);
};


/* **********************************************************
********************************************************** */
class DISPDU_fire_type {

   public:
      DIS_header_type header;

      unsigned short fire_site_id;
      unsigned short fire_application_id;
      unsigned short fire_entity_id;

      unsigned short target_site_id;
      unsigned short target_application_id;
      unsigned short target_entity_id;

      unsigned short ammo_site_id;
      unsigned short ammo_application_id;
      unsigned short ammo_entity_id;

      unsigned short event_site_id;
      unsigned short event_application_id;
      unsigned short event_entity_id;

      unsigned int fire_mission_index;
   
      vector3d location;

      DIS_burst_descriptor_type burst_data;

      vector3f velocity;

      float range;
   
      DISPDU_fire_type();
      virtual ~DISPDU_fire_type() {}

      void setup(int fire_id, DIS_id_type *target, int ammo_id,
                 int event_id, DIS_burst_descriptor_type *burst,
                 double *pos, double *vel, double *range,
                 unsigned int timer);
      void build(buffer_type *buffer);
      void extract(buffer_type *buffer);
};


/* **********************************************************
********************************************************** */
class DISPDU_detonate_type {

   public:
      DIS_header_type header;

      unsigned short fire_site_id;
      unsigned short fire_application_id;
      unsigned short fire_entity_id;

      unsigned short target_site_id;
      unsigned short target_application_id;
      unsigned short target_entity_id;

      unsigned short ammo_site_id;
      unsigned short ammo_application_id;
      unsigned short ammo_entity_id;

      // matches fire pdu or zeroed out
      unsigned short event_site_id;
      unsigned short event_application_id;
      unsigned short event_entity_id;

      vector3f velocity;

      vector3d location;

      DIS_burst_descriptor_type burst_data;
   
      vector3f location_target_coords;
   
      unsigned char detonation_result;
      unsigned char articulation_count;
      unsigned char padd[2];
   
      DISPDU_detonate_type();
      virtual ~DISPDU_detonate_type() {}

      void setup(DISPDU_fire_type *fire, int fire_id, DIS_id_type *target,
                 int ammo_id, int event_id,
                 DIS_burst_descriptor_type *burst, double *pos,
                 double *vel, double *target_local_pos, int det_result,
                 unsigned int timer);
      void build(buffer_type *buffer);
      void extract(buffer_type *buffer);
};


/* **********************************************************
********************************************************** */
class DISPDU_action_request_type {

   public:
      DIS_header_type header;

      unsigned short source_site_id;
      unsigned short source_application_id;
      unsigned short source_entity_id;

      unsigned short target_site_id;
      unsigned short target_application_id;
      unsigned short target_entity_id;

      unsigned int   request_id;
      unsigned int   action_id;
      unsigned int   fixed_record_count;
      unsigned int   variable_record_count;

      DISPDU_action_request_type();
      virtual ~DISPDU_action_request_type() {}

      void setup(DIS_id_type *source, DIS_id_type *target, int request_id,
                 int action_id, unsigned int timer);
      void build(buffer_type *buffer);
      void extract(buffer_type *buffer);
};


/* **********************************************************
********************************************************** */
class DISPDU_action_response_type {

   public:
      DIS_header_type header;

      unsigned short source_site_id;
      unsigned short source_application_id;
      unsigned short source_entity_id;

      unsigned short target_site_id;
      unsigned short target_application_id;
      unsigned short target_entity_id;

      unsigned int   request_id;
      unsigned int   request_status;
      unsigned int   fixed_record_count;
      unsigned int   variable_record_count;

      DISPDU_action_response_type();
      virtual ~DISPDU_action_response_type() {}
      
      void setup(DIS_id_type *source, DIS_id_type *target, int request_id,
                 int request_status, unsigned int timer);
      void build(buffer_type *buffer);
      void extract(buffer_type *buffer);
};


/* **********************************************************
********************************************************** */
class DIS_emissions_trackjam_type : public dbl_llist {

   public:
      unsigned short site_id;
      unsigned short application_id;
      unsigned short entity_id;
      unsigned char  emitter_id;
      unsigned char  beam_id;

      virtual ~DIS_emissions_trackjam_type() {}

      int  build(int *count, int *systemsize, int *beamsize, sfile *sbuffer);
      void extract(sfile *sbuffer);
};


/* **********************************************************
********************************************************** */
class DIS_emissions_beams_type : public dbl_llist {

   public:
//      unsigned char beam_size;
      unsigned char padd;
      unsigned char beam_id;
      unsigned short beam_parameter_index;

      float frequency;
      float freq_range;
      float power;
      float prf;
      float pulse_width;
      float beam_azimuth;
      float beam_az_sweep;
      float beam_elevation;
      float beam_ele_sweep;
      float beam_sweep_sync;

      unsigned char beam_function;
//      unsigned char target_count;
      unsigned char high_density;      
//      unsigned char padd2;
      unsigned char padd2[2];

      unsigned int jamming_mode;

      dbl_llist_manager trackjamman;      
 
      virtual ~DIS_emissions_beams_type() {}

      int  build(int *count, int *systemsize, sfile *sbuffer);
      void extract(sfile *sbuffer);
};

     
/* **********************************************************
********************************************************** */
class DIS_emissions_systems_type : public dbl_llist {

   public:   
//      unsigned char size;  // counts of 32 bit words
//      unsigned char beam_count;
//      unsigned short padd;

      unsigned short emitter_name;
      unsigned char emitter_function;
      unsigned char emitter_id;

      vector3f location;

      dbl_llist_manager beamman;


      DIS_emissions_systems_type() {
         emitter_name = DIS_EMITTER_NAME_OTHER;
         emitter_function = DIS_EMITTER_FUNCTION_OTHER;
         location[0] = location[1] = location[2] = 0;
      }

      virtual ~DIS_emissions_systems_type() {}

      int  build(int *count, sfile *sbuffer);
      void extract(sfile *sbuffer);
};


/* **********************************************************
********************************************************** */
class DISPDU_emissions_type {

   public:
      DIS_header_type header;

      unsigned short emitting_site_id;
      unsigned short emitting_application_id;
      unsigned short emitting_entity_id;

      unsigned short event_site_id;
      unsigned short event_application_id;
      unsigned short event_entity_id;

      unsigned char  state_update_indicator;
//      unsigned char  system_count;
//      unsigned char  padd[2];
      unsigned char padd[3];
      
      dbl_llist_manager systemman;
   
      DISPDU_emissions_type();
      virtual ~DISPDU_emissions_type() {}

      int  build(buffer_type *buffer);
      void extract(buffer_type *buffer);
      void setup(int id);
};


extern int dis_exercise_id;
extern int dis_site_id;
extern int dis_application_id;

void set_dis_interface(int exercise_id, int site_id, int app_id);
void get_dis_interface(int *exercise_id, int *site_id, int *app_id);

void init_dissweg(lla_type *center);
void init_disvissim(lla_type *center);

void local2dis(vector3d local, vector3d dis, int type);
void dis2local(vector3d dis, vector3d local, int type);

void local2disypr(vector3d localypr, vector3d disypr);
void dis2localypr(vector3d disypr, vector3d localypr);

void lla2dis(lla_type *in, vector3d out);
void dis2lla(vector3d in, lla_type *out);

unsigned int distime(double *seconds);

#endif

