

#include "vpixcon.h"
#include "anineutr.h"

camera   *current_camera = NULL;

gameatom *pixcon_object_list = NULL;               // atom list
gameatom *camera_tree  = NULL;
gameatom *enviro_tree  = NULL;
gameatom *terrain_tree = NULL;


/* **************************************************************
************************************************************** */
gameatom *load_atom(char *filename) {

   FILE *infile;
   gameatom *player;
   
   if (!find_file(filename, "r", OBJECT_PATH.string, PLATFORM_SLASH, NULL, &infile))
      return NULL;
      
   player = new gameatom;
   player->read_quark(infile, 0);

   fclose(infile);
   
   player->next = pixcon_object_list;
   return pixcon_object_list = player;
}


/* *************************************************************
************************************************************* */
int point_terrain_collision(quark *obstacle, quark *parent, double *start, double *end, double *midpt, double *radius, double *down, double *pos, double *normal) {

   linktype *ptr;
   vector4f startf, endf, midptf;
   vector4f posf, normalf;
   
   if ((obstacle->old_state.state_flags & STATE_MASK_TREE) == STATE_FLAG_TNONE)
      return 0;
      
   if ((obstacle->old_state.state_flags & STATE_MASK_TREE) == STATE_FLAG_TSPHERE &&
       (obstacle->old_state.tree_center[0] + obstacle->old_state.tree_radius < start[0] ||
        obstacle->old_state.tree_center[0] - obstacle->old_state.tree_radius > start[0] ||
        obstacle->old_state.tree_center[1] + obstacle->old_state.tree_radius < start[1] ||
        obstacle->old_state.tree_center[1] - obstacle->old_state.tree_radius > start[1]))
      return 0;

   copyarray3(startf, (float)start);
   startf[3] = 1.0;
   copyarray3(endf, (float)end);
   endf[3] = 1.0;
   copyarray3(midptf, (float)midpt);
   midptf[3] = 1.0;

   if ((obstacle->flags & QUARK_FLAG_ACTIVE) && (obstacle->flags & GAMEFLAG_PHYSICAL) &&
       ((antineutron *)obstacle)->obj_vs_lineseg(startf, endf, midptf, (float)*radius, posf, normalf, COL_POLYGON)) {

      copyarray3(pos, posf);
      pos[3] = 1.0f;
      copyarray4(normal, normalf);
      return 1;
   }

   // check w/ subtree
   for (ptr=(linktype *)obstacle->edge.head; ptr; ptr=(linktype *)ptr->next)
      if (ptr->link != parent && point_terrain_collision(ptr->link, obstacle, start, end, midpt, radius, down, pos, normal))
         return 1;

   return 0;
}


